###-----------------------------------------------------------------------------
### example: build_mcu_fpga_ip.tcl
### A joint work commissioned on behalf of SoC Labs, under Arm Academic Access license.
###
### Contributors
###
### David Flynn (d.w.flynn@soton.ac.uk)
###
### Copyright  2022, SoC Labs (www.soclabs.org)
###-----------------------------------------------------------------------------
#
# developed & tested using vivado_version 2021.1
#
# usage:
# vivado -mode tcl -source scripts/build_mcu_fpga_ip.tcl

#
# STEP#0: define output directory area.
#

set outputDir ./vivado/built_mcu_fpga
file mkdir $outputDir
#
# STEP#1: setup design sources and constraints
#

# local search path for configurations
set search_path ../verilog

set cortexm0_vlog    ../../../../../../arm-AAA-ip/Cortex-M0/AT510-BU-00000-r0p0-03rel3/logical
source scripts/rtl_source_cm0.tcl

set search_path [ concat $search_path $cortexm0_vlog/cortexm0_integration/verilog ]
read_verilog  [ glob $cortexm0_vlog/cortexm0_integration/verilog/*.v ]
read_verilog  [ glob $cortexm0_vlog/models/cells/*.v ]

# Arm unmodified CMSDK RTL
set cmsdk_vlog    ../../../../../../arm-AAA-ip/Corstone-101_Foundation_IP/BP210-BU-00000-r1p1-00rel0
source scripts/rtl_source_cmsdk.tcl

set search_path [ concat $search_path $cmsdk_vlog/logical/models/memories ]
read_verilog  $cmsdk_vlog/logical/models/memories/cmsdk_ahb_memory_models_defs.v
read_verilog  $cmsdk_vlog/logical/models/memories/cmsdk_ahb_rom.v
read_verilog  $cmsdk_vlog/logical/models/memories/cmsdk_fpga_rom.v
read_verilog  $cmsdk_vlog/logical/models/memories/cmsdk_ahb_ram.v
read_verilog  $cmsdk_vlog/logical/models/memories/cmsdk_fpga_sram.v

# configured Arm DMA-PL230 RTL (include ../verilog/pl230_defs.v for local configuration, not the distribution, already on search path)
set search_path [ concat $search_path ../verilog ]
set dma230_vlog    ../../../../../../arm-AAA-ip/DMA-230_MicroDMA_Controller/PL230-BU-00000-r0p0-02rel2/shared/logical/pl230_udma/verilog
source scripts/rtl_source_dma230.tcl

# ADP, FT1248 and streamio IP
source scripts/rtl_source_soclabs_ip.tcl

## FPGA-specific pads
#source scripts/rtl_source_fpga_ip.tcl

# soclabs modified mcu system 
set soc_vlog ../verilog
read_verilog  $soc_vlog/gen_ahb_busmatrix/verilog/built/soclabs_4x7_AhbMatrix/soclabs_4x7_AhbMatrix_default_slave.v
read_verilog  $soc_vlog/gen_ahb_busmatrix/verilog/built/soclabs_4x7_AhbMatrix/soclabs_4x7_AhbMatrix_lite.v
read_verilog  $soc_vlog/gen_ahb_busmatrix/verilog/built/soclabs_4x7_AhbMatrix/soclabs_4x7_AhbMatrix.v
read_verilog  $soc_vlog/gen_ahb_busmatrix/verilog/built/soclabs_4x7_AhbMatrix/soclabs_4x7_Arbiter.v
read_verilog  $soc_vlog/gen_ahb_busmatrix/verilog/built/soclabs_4x7_AhbMatrix/soclabs_4x7_MasterInput.v
read_verilog  $soc_vlog/gen_ahb_busmatrix/verilog/built/soclabs_4x7_AhbMatrix/soclabs_4x7_MatrixDecode_adp.v
read_verilog  $soc_vlog/gen_ahb_busmatrix/verilog/built/soclabs_4x7_AhbMatrix/soclabs_4x7_MatrixDecode_cpu.v
read_verilog  $soc_vlog/gen_ahb_busmatrix/verilog/built/soclabs_4x7_AhbMatrix/soclabs_4x7_MatrixDecode_dma2.v
read_verilog  $soc_vlog/gen_ahb_busmatrix/verilog/built/soclabs_4x7_AhbMatrix/soclabs_4x7_MatrixDecode_dma.v
read_verilog  $soc_vlog/gen_ahb_busmatrix/verilog/built/soclabs_4x7_AhbMatrix/soclabs_4x7_SlaveOutput.v
read_verilog  $soc_vlog/ahb_bootrom.v
read_verilog  $soc_vlog/bootrom.v
read_verilog  $soc_vlog/cmsdk_ahb_cs_rom_table.v
read_verilog  $soc_vlog/cmsdk_apb_usrt.v
##read_verilog  $soc_vlog/cmsdk_clkreset.v
read_verilog  $soc_vlog/cmsdk_ft1248x1_adpio.v
read_verilog  $soc_vlog/cmsdk_mcu_clkctrl.v
read_verilog  $soc_vlog/cmsdk_mcu_pin_mux.v
read_verilog  $soc_vlog/cmsdk_mcu_stclkctrl.v
read_verilog  $soc_vlog/cmsdk_mcu_sysctrl.v
##read_verilog  $soc_vlog/cmsdk_uart_capture.v
read_verilog  $soc_vlog/soclabs_ahb_aes128_ctrl.v
read_verilog  $soc_vlog/nanosoc_cpu.v
read_verilog  $soc_vlog/nanosoc_sys_ahb_decode.v
read_verilog  $soc_vlog/nanosoc_sysio.v
read_verilog  $soc_vlog/nanosoc_chip.v
read_verilog  $soc_vlog/nanosoc_chip_pads.v

set_property top nanosoc_chip [current_fileset]

# FPGA specific timing constraints
#read_xdc target_fpga/fpga_timing.xdc

## FPGA board specific pin constraints
#read_xdc target_fpga/fpga_pinmap.xdc

#
# STEP#2: run synthesis, report utilization and timing estimates, write checkpoint design
#

update_compile_order -fileset sources_1

set mculib_ip  $outputDir/MCULIB

ipx::package_project -root_dir $mculib_ip -vendor soclabs.org -library user -taxonomy /UserIP -import_files -set_current false -force -force_update_compile_order

ipx::unload_core  $mculib_ip/component.xml
ipx::edit_ip_in_project -upgrade true -name tmp_edit_project -directory  $mculib_ip  $mculib_ip/component.xml

update_compile_order -fileset sources_1
set_property ipi_drc {ignore_freq_hz true} [ipx::current_core]
ipx::merge_project_changes files [ipx::current_core]

set_property core_revision 2 [ipx::current_core]
ipx::update_source_project_archive -component [ipx::current_core]
ipx::create_xgui_files [ipx::current_core]
ipx::update_checksums [ipx::current_core]
ipx::check_integrity [ipx::current_core]

ipx::save_core [ipx::current_core]
ipx::check_integrity -quiet -xrt [ipx::current_core]
ipx::archive_core  $mculib_ip/soclabs.org_user_nanosoc_chip_1.0.zip [ipx::current_core]
ipx::move_temp_component_back -component [ipx::current_core]
#close_project -delete
close_project

set_property  ip_repo_paths { ip_repo $mculib_ip} [current_project]
update_ip_catalog
close_project
